@@ -18,10 +18,17 @@
 #' @export
 Ops.stars <- function(e1, e2) {
 	#ret = if (is.array(e2))
-	ret = if (!inherits(e2, "stars"))
+	ret = if (missing(e2))
+			lapply(e1, .Generic)
+		else if (!inherits(e2, "stars"))
 			lapply(e1, .Generic, e2 = e2)
-		else
-			mapply(.Generic, e1, e2, SIMPLIFY = FALSE)
+		else {
+			if (!all(dim(e1) == dim(e2))) {
+				stopifnot(length(e2) == 1)
+				lapply(e1, .Generic, e2 = as.vector(e2[[1]]))
+			} else
+				mapply(.Generic, e1, e2, SIMPLIFY = FALSE)
+		}
 	if (! inherits(e1, "stars"))
 		setNames(st_as_stars(ret, dimensions = st_dimensions(e2)), names(e2))
 	else
@@ -75,6 +82,8 @@ st_apply = function(X, MARGIN, FUN, ...) UseMethod("st_apply")
 #' @param ... arguments passed on to \code{FUN}
 #' @param CLUSTER cluster to use for parallel apply; see \link[parallel]{makeCluster}
 #' @param PROGRESS logical; if \code{TRUE}, use \code{pbapply::pbapply} to show progress bar
+#' @param FUTURE logical;if \code{TRUE}, use \code{future.apply::future_apply} 
+#' @param rename logical; if \code{TRUE} and \code{X} has only one attribute and \code{FUN} is a simple function name, rename the attribute of the returned object to the function name
 #' @return object of class \code{stars} with accordingly reduced number of dimensions; in case \code{FUN} returns more than one value, a new dimension is created carrying the name of the function used; see the examples.
 #' @examples
 #' tif = system.file("tif/L7_ETMs.tif", package = "stars")
@@ -84,8 +93,12 @@ st_apply = function(X, MARGIN, FUN, ...) UseMethod("st_apply")
 #' st_apply(x, 3, mean)   # mean of all pixels for each band
 #' st_apply(x, "band", mean) # equivalent to the above
 #' st_apply(x, 1:2, range) # min and max band value for each pixel
+#' # to get a progress bar also in non-interactive mode, specify:
+#' if (require(pbapply)) { # install it, if FALSE
+#'   pboptions(type = "timer")
+#' }
 #' @export
-st_apply.stars = function(X, MARGIN, FUN, ..., CLUSTER = NULL, PROGRESS = FALSE) {
+st_apply.stars = function(X, MARGIN, FUN, ..., CLUSTER = NULL, PROGRESS = FALSE, FUTURE = FALSE, rename = TRUE) {
 	fname <- paste(deparse(substitute(FUN), 50), collapse = "\n")
 	if (is.character(MARGIN))
 		MARGIN = match(MARGIN, names(dim(X)))
@@ -93,15 +106,22 @@ st_apply.stars = function(X, MARGIN, FUN, ..., CLUSTER = NULL, PROGRESS = FALSE)
 
 	if (PROGRESS && !requireNamespace("pbapply", quietly = TRUE))
 		stop("package pbapply required, please install it first")
+	
+	if (FUTURE && !requireNamespace("future.apply", quietly = TRUE))
+	  stop("package future.apply required, please install it first")
 
 	fn = function(y, ...) {
 		ret = if (PROGRESS)
-				pbapply::pbapply(y, MARGIN, FUN, ..., cl = CLUSTER)
+				pbapply::pbapply(X = y, MARGIN = MARGIN, FUN = FUN, ..., cl = CLUSTER)
 			else {
-				if (is.null(CLUSTER))
-					apply(y, MARGIN, FUN, ...)
-				else
-					parallel::parApply(CLUSTER, y, MARGIN, FUN, ...)
+				if (is.null(CLUSTER) && !FUTURE)
+					apply(X = y, MARGIN = MARGIN, FUN = FUN, ...)
+				else if (FUTURE) {
+					oopts = options(future.globals.maxSize = +Inf)
+					on.exit(options(oopts))
+					future.apply::future_apply(y, MARGIN = MARGIN, FUN = FUN, ...)
+				} else
+					parallel::parApply(CLUSTER, X = y, MARGIN = MARGIN, FUN = FUN, ...)
 			}
 		if (is.array(ret))
 			ret
@@ -110,16 +130,32 @@ st_apply.stars = function(X, MARGIN, FUN, ..., CLUSTER = NULL, PROGRESS = FALSE)
 	}
 	ret = lapply(X, fn, ...) 
 	dim_ret = dim(ret[[1]])
-	if (length(dim_ret) == length(MARGIN)) # FUN returned a single value
-		st_stars(ret, st_dimensions(X)[MARGIN])
-	else { # FUN returned multiple values:
-		orig = st_dimensions(X)[MARGIN]
-		r = attr(orig, "raster")
-		dims = c(structure(list(list()), names = fname), orig)
-		dims[[1]] = if (!is.null(dimnames(ret[[1]])[[1]])) # FUN returned named vector:
-				create_dimension(values = dimnames(ret[[1]])[[1]])
-			else
-				create_dimension(to = dim_ret[1])
-		st_stars(ret, dimensions = create_dimensions(dims, r))
-	}
+	ret = if (length(dim_ret) == length(MARGIN)) { # FUN returned a single value
+			if (length(ret) == 1 && rename && make.names(fname) == fname)
+				ret = setNames(ret, fname)
+			st_stars(ret, st_dimensions(X)[MARGIN])
+		} else { # FUN returned multiple values: need to set dimension name & values
+			dim_no_margin = dim(X)[-MARGIN]
+			no_margin = setdiff(seq_along(dim(X)), MARGIN)
+			if (length(no_margin) > 1 && dim(ret[[1]])[1] == prod(dim_no_margin)) {
+				r = attr(st_dimensions(X), "raster")
+				new_dim = c(dim_no_margin, dim(ret[[1]])[-1])
+				for (i in seq_along(ret))
+					dim(ret[[i]]) = new_dim
+				# set dims:
+				dims = st_dimensions(X)[c(no_margin, MARGIN)]
+			} else {
+				orig = st_dimensions(X)[MARGIN]
+				r = attr(orig, "raster")
+				dims = c(structure(list(list()), names = fname), orig)
+				dims[[1]] = if (!is.null(dimnames(ret[[1]])[[1]])) # FUN returned named vector:
+						create_dimension(values = dimnames(ret[[1]])[[1]])
+					else
+						create_dimension(to = dim_ret[1])
+			}
+			st_stars(ret, dimensions = create_dimensions(dims, r))
+		}
+	for (i in seq_along(ret))
+		names(dim(ret[[i]])) = names(st_dimensions(ret))
+	ret
 }
